# File: ~/.quirc/global.tcl
# This file contains any tcl that is intended to be loaded globally.  For
# server specific tcl, see default.tcl.


# ALIASES

# /abort
#     Exits QuIRC.
alias abort {
    exit
}

# /alias <name> <script>
#     Creates an alias in the dynamic namespace.  It creates wrapper code so
#     that if the alias is executed from a server as opposed to globally, it
#     will run as if it were a server specific alias.  Here is a simple
#     example that allows you to use /new_cd to activate /cd:
#         /alias /new_cd /cd $arg
#     or
#         /alias new_cd /cd $arg
#     script can be as complicated as you wish, there is no need to limit it to
#     something as simple as in the examples.
alias alias {
    set name [lindex [split $arg " "] 0]
    if { [string index $name 0]=="/" } {
	set name [string range $name 1 end]
    }
    set script [join [lrange [split $arg " "] 1 end]]
    if { $arg==" " || $arg=="" } {
	echo " \0030,4 SCRIPT \003 Dynamic Aliases:"
	set aliaslist ""
	foreach command [info commands ::dynamic::alias_*] {
	    lappend aliaslist [string range [lindex [split $command ":"] 4] 6 end]
	}
	echo " \0030,4 SCRIPT \003 [join $aliaslist]"
    } else {
	if { [string index $name 0]=="-" } {
	    set name [string range $name 1 end]
	    if { [string index $name 0]=="/" } {
		set name [string range $name 1 end]
	    }
	    rename ::dynamic::alias_$name ""
	    echo " \0030,4 SCRIPT \003 Removing alias $name from ::dynamic namespace."
	} else {
	    proc ::dynamic::alias_$name { arg } "
                set server -1
                if { \[info level]==1 } {
                    if { \[currentindex]!=-1 } {
	                set server \[currentindex]
                    }
                } else {
	            if { \[info level] > 2 } {
                        set server \[lindex \[split \[info level -2] \"::\"] 2]
	                if {!\[regexp \"\\\[0-9]+\" \$server]} {set server -1}
                    }
                }
                if { \$server==-1 } {
	            $script
                } else {
                    while { \[info commands \[set procname ::\${server}::internal::evalserver_\[expr rand()]]]!=\"\" } {}
                    proc \$procname { arg } {
                        $script
                    }
                    \$procname \$arg
                    rename \$procname \"\"
                }
            "
	    echo " \0030,4 SCRIPT \003 Creating alias $name in ::dynamic namespace."
	}
    }
}

# /append <variable> [<value>]
#     Appends onto the list given by variable (in the dynamic namespace) the
#     given value.  If no value is given, shows the joined contents of the
#     variable.
alias append {
    set args [split $arg]
    set data [join [lrange [split $arg " "] 1 end]]
    if { ![llength $args] } {
	error "Usage: /append <variable> [<value>]"
    }
    if { [llength $args]>=2 } {
	echo " \0030,6 SET \003\0032 [lindex $args 0] \003set to:\0032 [join [lappend ::dynamic::[lindex $args 0] $data]]"
    } else {
	echo " \0030,6 SET \003\0032 [lindex $args 0] \003is set to:\0032 [join [set ::dynamic::[lindex $args 0]]]"
    }
}

# /beep
#     Rings the bell.
alias beep {
    bell
}

# /bye [<message>]
#     Exits the client sending quit message given to all the current servers.
#     The quit message does not always arrive at the servers, sometimes the
#     socket is closed too soon.
alias bye {
    if { $arg!="" } {
	foreach server [servers] {
	    evalserver $server "quote \"QUIT :$arg\""
	}
    }
    exit
}

# /cd [<dir>]
#     Changes or displays the current working directory of the client.
alias cd {
    if { $arg=="" } {
	echo " \0030,14 DIR \003 Current: [pwd]"
    } else {
	cd $arg
	echo " \0030,14 DIR \003 Changed to: $arg"
    }
}

# /clear
#     Clears the text from the current window.
alias clear {
    [currentwindow].text configure -state normal
    [currentwindow].text delete 0.0 end
    [currentwindow].text configure -state disabled
    echo [currentwindow] 0 $::pre_text_insertion
}

# /doc [<tcl file>]
#     Displays documentation for the given file in the Main Status window.
#     If no file is given, it lists the tcl files in the directory.
alias doc {
    if { $arg=="" } {
	echo "\0032\002***** ~/.quirc/*.tcl *****"
	set list ""
	foreach file [glob ~/.quirc/*.tcl] {
	    echo [lindex [split $file "/"] end]
	}
	echo "\0032\002***** End of List *****"
	return
    }
    set fh [open "$::env(HOME)/.quirc/$arg"]
    echo "\0032\002***** $arg Documentation *****"
    set inarea 0
    while { ![eof $fh] } {
	gets $fh theline
	if { [string range $theline 0 1]=="# " } {
	    set text [string range $theline 2 end]
	    if { [string toupper $text]==$text } {
		echo "\0034\002$text"
		set inarea 1
	    } elseif { [string index $text 0]==" " } {
		echo $text
	    } else {
		if { $text=="alias <name> <script>" } { set inarea 1 }
		if { $inarea } {
		    echo "\0032[lindex [split $text " "] 0] \0033[join [lrange [split $text " "] 1 end]]"
		} else {
		    echo "$text"
		}
	    }
	}
    }
    close $fh
    echo "\0032\002***** End of Documentation *****"
}

# /eval <tcl script>
#     Evaluates the given tcl script in the global namespace if in the Main
#     Status window, otherwise it evaluates in the namespace of the current
#     server.  The results are echoed to the current window.
alias eval {
    if { [currentwindow]==".main" } {
	uplevel "#0" "echo \[eval {$arg}]"
    } else {
	#puts "About to run it!"
	#set data [evalserver [currentindex] "\[eval {$arg}]"]
	#puts "Got data... it is:"
	#puts $data
	evalserver [currentindex] "echo \[eval {$arg}]"
    }
}

# /evalglobal <tcl script>
#     Evalutates the given tcl script in the global namespace.  The results are
#     echoed to the current window.
alias evalglobal {
    uplevel "#0" "echo \[eval {$arg}]"
}

# /evalglobalnr <tcl script>
#     Evalutates the given tcl script in the global namespace.  The results are
#     not echoed.
alias evalglobalnr {
    uplevel "#0" "eval {$arg}"
}

# /evalnr <tcl script>
#     Evaluates the given tcl script in the global namespace if in the Main
#     Status window, otherwise it evaluates in the namespace of the current
#     server.  The results are not echoed.
alias evalnr {
    if { [currentwindow]==".main" } {
	uplevel "#0" "eval {$arg}"
    } else {
	evalserver [currentindex] "eval {$arg}"
    }
}

# /load <script>
#     Loads the given script either into the global namespace if the Main
#     Status window is presently active, or into the regular script namespace
#     of the active server.
alias load {
    if { [currentindex]=="-1" } {
	scriptglobal $arg
    } else {
	evalserver [currentindex] "script $arg"
    }
}

# /loadglobal <script>
#     Loads the given script into the global namespace.  Note that there are no
#     provisions in place for easily unloading globally loaded scripts.
alias loadglobal {
    scriptglobal $arg
}

# /newserver [<server>[:[<port>][:<password>]]]
#     Starts a new server using the given server, port, and password data.
alias newserver {
    eval "newserver [split $arg]"
}

# /rawview
#     Allows you to view the output to and from the server.
alias rawview {
    raise .raw
}

# /remove <variable> <value>
#     Removes from the list given by variable (in the dynamic namespace) the
#     given value.
alias remove {
    set args [split $arg " "]
    if { [llength $args]<2 } {
	error "Usage: /remove <variable> <value>"
    }
    set data [join [lrange $args 1 end]]
    set index [lsearch -exact [set ::dynamic::[lindex $args 0]] $data]
    if { $index==-1 } {
	error "$data not found in variable ::dynamic::[lindex $args 0]"
    }
    set ::dynamic::[lindex $args 0] [lreplace [set ::dynamic::[lindex $args 0]] $index $index]
    echo " \0030,6 SET \003\0032 [lindex $args 0] \003set to:\0032 [join [set ::dynamic::[lindex $args 0]]]"
}

# /server [<server>[:[<port>][:<password>]]]
#     Starts a new server using the given server, port, and password data.
alias server {
    newserver $arg
}

# /set [-|+][<variable>] [<value>]
#     Stores value in the given variable in the dynamic namespace.  If no value
#     is given, it will print the contents of the variable.  If no arguments
#     at all are given, it will show a sorted list of all variables and their
#     values.  If the variable is prefixed with -, it will set the variable to
#     empty.  If the variable is prefixed with +, it will be appended to.
#     Set allows wildcards for the displaying of variables.
alias set {
    set args [split $arg " "]
    set data [join [lrange [split $arg " "] 1 end]]
    set glob [lindex $args 0]
# Make better array viewing (erroneous code at bottom)
    if { [lindex $args 0]=="" } {
	foreach var [lsort [info vars ::dynamic::*]] {
	    if { [array exists $var] } {
		foreach name [lsort [array names $var]] {
		    echo " \0030,6 SET \003\0032 [string range $var 11 end]($name) \003is set to\0032 [set ${var}($name)]"
		}
	    } else {
		echo " \0030,6 SET \003\0032 [string range $var 11 end] \003is set to\0032 [set $var]"
	    }
	}
    } else {
	if { $data!="" || [string index [lindex $args 0] 0]=="-" || ([string index [lindex $args 0] 0]=="+" && $data!="") } {
	    if { [string index [lindex $args 0] 0]=="-" } {
		set ::dynamic::[string range [lindex $args 0] 1 end] ""
		echo " \0030,6 SET \003\0032 [string range [lindex $args 0] 1 end] \003cleared."
	    } elseif { [string index [lindex $args 0] 0]=="+" } {
		set varname [string range [lindex $args 0] 1 end]
		set ::dynamic::$varname "[set ::dynamic::$varname]$data"
		echo " \0030,6 SET \003\0032 $varname \003appended to be \0032[set ::dynamic::$varname]"
	    } else {
		set ::dynamic::[lindex $args 0] $data
		echo " \0030,6 SET \003\0032 [lindex $args 0] \003set to\0032 $data"
	    }
	} else {
	    foreach variable [lsort [info vars ::dynamic::[lindex $args 0]]] {
		if { [array exists $variable] } {
		    foreach key [lsort [array names $variable]] {
			echo " \0030,6 SET \003\0032 [string range $variable 11 end]($key) \003is set to\0032 [set ${variable}($key)]"
		    }
		} else {
		    echo " \0030,6 SET \003\0032 [string range $variable 11 end] \003is set to\0032 [set $variable]"
		}
	    }
	}
    }
}

# /store [-geometry|-size]
#     Saves the data stored in the dynamic namespaces into dynamic.tcl.  If
#     -geometry is specified, then the current size and position of the QuIRC
#     window will be saved.  If -size is specified, then only the size will be
#     stored.  The old dynamic.tcl will be backed up as dynamic.tcl~.
alias store {
    if { [file exists $::env(HOME)/.quirc/dynamic.tcl] } {
	file copy -force "$::env(HOME)/.quirc/dynamic.tcl" "$::env(HOME)/.quirc/dynamic.tcl~"
    }
    set of [open $::env(HOME)/.quirc/dynamic.tcl "WRONLY CREAT TRUNC"]
    if { $arg=="-geometry" } {
	echo " \0030,4 SCRIPT \003 Setting defaultgeometry to [wm geometry .]"
	set ::dynamic::defaultgeometry [wm geometry .]
    } elseif { $arg=="-size" } {
	echo " \0030,4 SCRIPT \003 Setting defaultgeometry to [lindex [split [wm geometry .] "+"] 0]"
	set ::dynamic::defaultgeometry [lindex [split [wm geometry .] "+"] 0]
    } elseif { $arg!="" } {
	error "usage: /store [-geometry|-size]"
    }
    foreach var [info vars ::dynamic::*] {
	if { ![array exists $var] } {
	    puts $of "set $var [escape [set $var]]"
	} else {
	    puts $of "array set $var {[array get $var]}"
	}
    }
    foreach command [info commands ::dynamic::alias_*] {
	set procname [lindex [split $command ":"] 4]
	puts $of "proc $procname { [info args $command] } { [info body $command] }"
    }
    close $of
    echo " \0030,4 SCRIPT \003 Storing ::dynamic namepace to ~/.quirc/dynamic.tcl"
}

# /unset <variable> [<variable> ...]
#     Stores value in the given variable in the dynamic namespace.  If no value
#     is given, it will print the contents of the variable.  If no arguments
#     all all are given, it will show available variables.
alias unset {
    if { $arg=="" } {
	set varlist ""
	foreach var [info vars ::dynamic::*] {
	    lappend varlist [string range $var 11 end]
	}
	echo " \0030,6 SET \003 Available variables: $varlist"
    } else {
	foreach var [split $arg " "] {
	    unset ::dynamic::$var
	    echo " \0030,6 SET \003\0032 $var \003unset."
	}
    }
}

# EVENTS

# event_client_start
#     This event is called after all of the normal initialization has occured.
#     It will not be called if the first_time variable is set.  In the default
#     version, it checks if there is an array called auto_start, set by
#     commands such as:
#         /set auto_start(irc.server1.net) /join #Channel1,#Channel2
#         /set auto_start(irc.server2.net) /join #Channel3 Key
#     It will start the servers specified and run the given script.  The
#     script should be valid TCL script and can contain semi-colons to seperate
#     multiple commands.  If the array doesn't exist, newserver will be run
#     with the default server and port settings.  The server set in the
#     auto_start array can have the same format as /newserver allows.
proc event_client_start {} {
    if { [array exists ::dynamic::auto_start] } {
	foreach server [array names ::dynamic::auto_start] {
	    set ::[newserver $server]::auto_start [set ::dynamic::auto_start($server)]
	}
    } else {
	newserver
    }
}

# event_first_time
#     This event is called when the client is started if the first_time
#     variable is set.
proc event_first_time {} {
    echo "***** QuIRC New User Instructions *****"
    echo "- Since this is your first time running QuIRC, here are some instructions to get you going."
    echo "- First, right click in this window and choose \"Basic Configuration\" to set certain basic variables.  If you need descriptions for these variables, you can either pick \"Configuration Variables\" from the \"Help\" menu, or use /doc variables.tcl."
    echo "- Once you have chosen appropriate settings, pick one of the options from the \"Store\" menu.  These values could also be stored with /store."
    echo "- The next time you start up QuIRC, the default server you specified will be started automatically.  For now, you can either start a new server by clicking on \"Start a New Server\" (Alt-n) or by typing /newserver irc.myserver.org."
    echo "- Good luck, and if you encounter problems, feel free to try #QuIRC on DALnet."
}

# event_windowchanged
#     Is called whenever the user switches windows.  It currently changes the
#     server lag indicator to match the showing server.
proc event_windowchanged { oldpathname oldname oldserver oldindex pathname name server index } {
    if { [winfo exists .buttonbar.lag] } {
	if { $server==-1 } {
	    .buttonbar.lag configure -text ""
	} else {
	    if { [info vars ::${server}::lagtext]!="" } {
		.buttonbar.lag configure -text [set ::${server}::lagtext]
	    } else {
		.buttonbar.lag configure -text ""
	    }
	}
    }
    if { [string match ".query*" $pathname] } {
	evalserver $server "
	    if { \[userhost [escape $name]]==\"\" } {
                title $pathname \"QuIRC - [escape $name]\"
            } else {
                title $pathname \"QuIRC - [escape $name] (\[userhost [escape $name]])\"
            }
	"
    }
}

# PROCEDURES

# evalserver <index> <script>
#     Makes the given script run under the server given by index.
#     Is equivalent to: namespace eval ::<index> <script>
proc evalserver { index script } {
    return [namespace eval ::$index $script]
}

# fact <number>
#     A function that calculates the factorial of a number using recursion.
#     If you need it to calculate large imprecise factorials, use a floating
#     point number (31 is the integer limit, 170.0 is the float limit).
proc fact { n } {
    if { $n<2 } {
	return 1
    } else {
	return [expr $n*[fact [expr $n-1]]]
    }
}

# Old evalserver code
#while { [info commands [set procname ::${index}::internal::evalserver_[expr rand()]]]!="" } {}
#proc $procname {}  "$script"
#$procname
#rename $procname ""

# is_ip_addr <ip>
#     Returns true if ip is a numeric dotted ipv4 address.
proc is_ip_addr { addr } {
    return [regexp {^[0-9][0-9]?[0-9]?\.[0-9][0-9]?[0-9]?\.[0-9][0-9]?[0-9]?\.[0-9][0-9]?[0-9]?$} $addr]
}

# is_number <number>
#     Returns true if the given argument is a number, false if not.
proc is_number { number } {
    if {[catch [list expr "double($number)"]]} {
	return 0
    } else {
	return 1
    }
}

# llongest <list>
#     Returns the string length of the longest entry in the list.
proc llongest { list } {
    set longest 0
    foreach entry $list {
	if { [string length $entry] > $longest } {
	    set longest [string length $entry]
	}
    }
    return $longest
}

# mask <type> <mask>
#     Returns a generalized version of the given mask according to the given
#     type.  To see what types there are, view the return lines at the bottom
#     with the following in mind:  $n is nick, $u is user, $h is host, $d is
#     the host with a * mask as the first of last section depending on if it's
#     numeric or alphabetic.
proc mask { type mask } {
    set n "*"
    set u "*"
    set a "*"
    scan $mask "%\[^!\]!%\[^@\]@%s" n u a
    set n [string trimleft $n "@+"]
    set u [string trimleft $u "~"]
    set h $a
    set d ""
    if { [is_ip_addr $a] } {
        set a [split $a .]
        set a [lreplace $a end end *]
    } else {
        set a [split $a .]
        if { [llength $a] > 2 } { set a [lreplace $a 0 0 *] }
    }
    set d [join $a .]
    if { [string length $u] >= 10 } {
	set u [string range $u [expr [string length $u]-9] end]
    }
    switch "$type" {
        "0" { return "*!$u@$h" }
        "1" { return "*!*$u@$h" }
        "2" { return "*!*@$h" }
        "3" { return "*!*$u@$d" }
        "4" { return "*!*@$d" }
        "5" { return "$n!$u@$h" }
        "6" { return "$n!*$u@$h" }
        "7" { return "$n!*@$h" }
        "8" { return "$n!*$u@$d" }
        "9" { return "$n!*@$d" }
    }
    return "$n!$u@$h"
}

# max <a> <b>
#     Returns a unless b is larger than a, in which case it returns b.
proc max { a b } {
    if { $b > $a } {
	return $b
    } else {
	return $a
    }
}

# push <stack> <item>
#     Pushes the item into the stack.
proc push { thestack item } {
    upvar $thestack stack
    if { ![info exists stack] } {
	set stack ""
    }
    set stack [linsert $stack 0 $item]
}

# pop <stack>
#     Pops an item off the given stack.
proc pop { thestack } {
    upvar $thestack stack
    set temp [lindex $stack 0] 
    set stack [lrange $stack 1 end]
    return $temp
}

# rpn <arg1> <arg2> ...
#     Returns the results from evaluating the given rpn expression.
proc rpn { args } {
    foreach arg $args {
	if { [is_number $arg] } {
	    push stack $arg
	} else {
	    set second [pop stack]
	    set first [pop stack]
	    push stack [expr $first $arg $second]
	}
    }
    return [pop stack]
}

# scriptglobal <script>
#     Loads the given script into the global namespace.
proc scriptglobal { arg } {
    uplevel "#0" "source \"\$env(HOME)/.quirc/$arg\""
    echo " \0030,4 SCRIPT \003 Loading $arg into global namespace."
}

# unknown <command> [<arg(s)>]
#     Handles commands which aren't found and handled immediately.
proc unknown { args } "
    set server \[lindex \[split \[uplevel {namespace current}] :] 2]
    if { !\[regexp {\[0-9]+} \$server] } {
        set server {}
    }
    set command \[lindex \$args 0]
    if { \$server!={} } {
        if { \[string index \$command 0]==\"/\" } {
            set command \[string range \$command 1 \[expr \[string length \$command]-1]]
            set parsed 0
            set returnvalue 0
            foreach script \[set ::\${server}::scripts] {
                if { \"\[info commands ::\${server}::\${script}::alias_\$command]\"!=\"\" } {
                    set parsed 1
                    set returnvalue \[::\${server}::\${script}::alias_\$command \[join \[lrange \$args 1 end]]]
                    if { \$returnvalue==1 } {
                        break
                    } else {
                        set returnvalue 0
                    }
                }
            }
            if { !\$returnvalue && \"\[info commands ::dynamic::alias_\$command]\"!=\"\" } {
                set parsed 1
                set returnvalue \[::dynamic::alias_\$command \[join \[lrange \$args 1 end]]]
                if { \$returnvalue!=1 } {
                    set returnvalue 0
                }
            }
            if { !\$returnvalue && \"\[info commands ::alias_\$command]\"!=\"\" } {
                set parsed 1
                ::alias_\$command \[join \[lrange \$args 1 end]]
            }
            if { !\$parsed } {
                ::template::quote \$server \"\$command \[join \[lrange \$args 1 end]]\"
            }
        } else {
            if { \"\[info commands ::template::\$command]\"!=\"\" } {
                return \[callproc ::template::\$command \[lrange \$args 1 end] \$server]
            } else {
                [info body unknown_original]
            }
        }
    } else {
        if { \[string index \$command 0]==\"/\" } {
            set command \[string range \$command 1 \[expr \[string length \$command]-1]]


            set returnvalue 0
            set parsed 0
            if { \"\[info commands ::dynamic::alias_\$command]\"!=\"\" } {
                set parsed 1
                set returnvalue \[::dynamic::alias_\$command \[join \[lrange \$args 1 end]]]
                if { \$returnvalue!=1 } {
                    set returnvalue 0
                }
            }
            if { !\$returnvalue && \"\[info commands ::alias_\$command]\"!=\"\" } {
                set parsed 1
                ::alias_\$command \[join \[lrange \$args 1 end]]
            }
            if { !\$parsed } {
                [info body unknown_original]
            }
        } else {
            [info body unknown_original]
        }
    }
"

#- Old Unknown Code.  Went at the top of the unknown code.
#-    if { \[info level]>1 } {
#-        set server \[lindex \[split \[info level -1] \"::\"] 2]
#-    } else {
#-        set server \"\"
#-    }
#-    if { \[regexp \"\\\[0-9]+\" \$server] } {

# SERVER SPECIFIC PROCEDURES

#- First not channel, set channel currentchannel.
#- 3 arguments, go for it.

# ban [<channel>] <nick or mask> [<type>]
#    Bans the given nick from the given, or current, channel.  If no type is
#    given, type 3 (*!*user@*.domain.name) is assumed.  If the nick contains
#    the ! character, it will simply ban it, ignoring any given type.
proc ::template::ban { index args } {
    if { [llength $args]>3 } {
	error "usage: ban [<channel>] <nick> [<type>]"
    }
    set channel [lindex $args 0]
    set nick [lindex $args 1]
    set type [lindex $args 2]
    if { [llength $args]==1 } {
	set channel [channel]
	set nick [lindex $args 0]
	set type 3
    }
    if { [llength $args]==2 } {
	if { [ischannel [lindex $args 0]] } {
	    set type 3
	} else {
	    set channel [channel]
	    set nick [lindex $args 0]
	    set type [lindex $args 1]
	}
    }
    if { [string match *!* $nick] } {
	::template::quote $index "MODE $channel +b $nick"
    } else {
	::template::quote $index "MODE $channel +b [mask $type $nick![::template::userhost $index $nick]]"
    }
}

proc ::template::modequeue { index command target args } {
    set target [string tolower $target]
    if { ![info exists ::${index}::modequeue($target)] } {
	set ::${index}::modequeue($target) ""
    }
    if { ![info exists ::${index}::modeparam($target)] } {
	set ::${index}::modeparam($target) ""
    }
    switch -exact -- $command {
	"add" {
	    if { [llength $args]<1 || [llength $args]>2 } {
		error "usage: modequeue (flush|add <target> <mode> [parameter])"
	    }
	    if { [lindex $args 1]!="" } {
		lappend ::${index}::modeparam($target) [lindex $args 1]
	    }
	    lappend ::${index}::modequeue($target) [lindex $args 0]
	    if { [llength [set ::${index}::modequeue($target)]] == $::dynamic::max_modes } {
		::template::modequeue ${index} flush $target
	    }
	}
	"flush" {
	    if { [llength $args]>0 } {
		error "usage: modequeue (flush|add <target> <mode> [parameter])"
	    }
	    if { [set ::${index}::modequeue($target)]!="" } {
		::template::quote $index "MODE $target [join [set ::${index}::modequeue($target)] {}] [join [set ::${index}::modeparam($target)]]"
		unset ::${index}::modequeue($target)
		unset ::${index}::modeparam($target)
	    }
	}
	default {
	    error "usage: modequeue (flush|add <target> <mode> [parameter])"
	}
    }
}

# queueban [<channel>] <nick or mask> [<type>]
#    Adds the ban for the given nick to the modequeue.  To actually send the
#    mode changes, do a modequeue flush #channel.  If no channel is given, the
#    current channel is used.  If no type is given, type 3
#    (*!*user@*.domain.name) is assumed.   If the nick contains the !
#    character, it will simply ban it, ignoring any given type.
proc ::template::queueban { index args } {
    if { [llength $args]>3 } {
	error "usage: ban [<channel>] <nick or mask> [<type>]"
    }
    set channel [lindex $args 0]
    set nick [lindex $args 1]
    set type [lindex $args 2]
    if { [llength $args]==1 } {
	set channel [channel]
	set nick [lindex $args 0]
	set type 3
    }
    if { [llength $args]==2 } {
	if { [ischannel [lindex $args 0]] } {
	    set type 3
	} else {
	    set channel [channel]
	    set nick [lindex $args 0]
	    set type [lindex $args 1]
	}
    }
    if { [string match *!* $nick] } {
	::template::modequeue $index add $channel +b $nick
    } else {
	::template::modequeue $index add $channel +b [mask $type $nick![::template::userhost $index $nick]]
    }
}

#-Some key bindings
bind . <Up> {histup}
bind . <Down> {histdown}
bind . <Control-Down> {nextwindow}
bind . <Control-Up> {previouswindow}
bind . <Shift-Prior> {[currentwindow].text yview scroll -1 page}
bind . <Shift-Next> {[currentwindow].text yview scroll 1 page}
bind . <Prior> {[currentwindow].text yview scroll -1 page}
bind . <Next> {[currentwindow].text yview scroll 1 page}
bind . <Control-Prior> {if { [channel]!="" } { [currentwindow].nicks yview scroll -1 page } }
bind . <Control-Next> {if { [channel]!="" } { [currentwindow].nicks yview scroll 1 page } }
bind . <Alt-Prior> {.windowlist.t yview scroll -1 page}
bind . <Alt-Next> {.windowlist.t yview scroll 1 page}
bind . <Control-F4> {closewindow [currentwindow]}
bind . <Alt-F4> {exit}
bind . <Alt-n> {source $env(HOME)/.quirc/newserver.tcl}
